<?php
require_once('request.php');

class arkiwiRequestPostRemoveMetadata extends arkiwiRequest
{
	public static function factory($ak) {
		if($ak->getRequestMethod() != 'POST')
			return null;

		if($ak->getRequestPath() == 'removemetadata' &&
				count($ak->getRequestExtra()) == 1) {
			$extra = $ak->getRequestExtra();
			return new arkiwiRequestPostRemoveMetadata($ak, $extra[0]);
		}

		return null;
	}

	// Post Metadata Object -------------------------

	private $sId = null;

	public function __construct($request) {
		parent::__construct($request);

		$extra = $this->getRequestExtra();

		$this->sId = $extra[0];
	}

	public function run() {
		if($this->request['method'] != 'POST') {
			$this->setBodyError(500, "arkiwiRequestPostRemoveMetadata: POST method required.");
			return;
		}

		if (!$ss->isValid($this->sId)) {
			$this->setBodyError(400, "Session invalid or expired.");
			return;
		}

		$input = file_get_contents('php://input');
		$obj = json_decode($input, true);
		if (!is_array($obj)) {
			$this->setBodyError(400, "Wrong json input");
			return;
		}

		foreach($obj as $kv) {
			if (isset($kv['key']) && isset($kv['value'])) {
				$this->removeKeyValue($this->sId, $kv['key'], $kv['value']);
			}
		}

		$this->setCode(201);
		$this->setBodyJSON(true);
	}

	private function removeKeyValue($session, $k, $v) {
		unset($_SESSION['keyValues'][$k]);
	}
}
